﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;    //串口控件
using System.Threading;

namespace CSI_MIPI_CLIENT
{
    public unsafe partial class Form1 : Form
    {
        public const int LEDCOUNTER=15;  // 共有15组 LED
        public int NoteIndex = 0;
        public int Elsp = 0;
        // 发送给FireBeetle 的数据 Buffer
        public byte[] Note = new byte[LEDCOUNTER];
        // MIDI 数据对应LED 序列号
        public int[] NoteToOrder = new int[LEDCOUNTER] 
                                { 91,89,88,86,84,       // 5# 4# 3# 2# 1#
                                  83,81,79,77,76,74,72, // 7 6 5 4 3 2 1
                                  71,69,67              // #7 #6 #5
        };
        public const int ONEDELAY = 150;  // 一个延时150ms
        // 记录音乐的结构体
        public struct nodeStruct
        {
            public int time;        //绝对时间，比如: 1200
            public int note;        //音符值， MIDI 的音符值
            public int delayunit;   //延时数量, 比如：3， 表示从上一个到这个音符延迟3个单位
        };
        // 记录整首音乐
        public List<nodeStruct> NoteList = new List<nodeStruct>();

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            if (System.IO.Ports.SerialPort.GetPortNames().Length != 0)
            {

                // List COM all ports in comboxBox1
                comboBox1.Items.AddRange(System.IO.Ports.SerialPort.GetPortNames());
                comboBox1.SelectedIndex = 0;

                serialPort1.BaudRate = 115200;
                serialPort1.DataBits = 8;
                serialPort1.Parity = 0;
                serialPort1.StopBits = (StopBits)1;
                serialPort1.Encoding = System.Text.Encoding.GetEncoding(28591);

                System.Windows.Forms.Control.CheckForIllegalCrossThreadCalls = false;

                // Show Application version in Form Title 
                this.Text = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString() + "\n";

            }
            else
            {
                textBox1.AppendText("[ERROR] No COM ports are found!");
                button1.Enabled = false;
            }
        }


        // 打开串口
        private void button1_Click(object sender, EventArgs e)
        {
            // 如果当前串口已经打开，那么就关闭之
            if (serialPort1.IsOpen)
            {
                //  停止定时器
                timer1.Enabled = false;
                // 更改按钮名称
                button1.Text = "Open Port";
                // 输出Log 
                textBox1.AppendText("Close Port at " + DateTime.Now.ToString() + "\r\n");
                serialPort1.Close();
            }
            else
            {
                // 如果串口未打开
                // 打开串口
                try
                {
                    serialPort1.PortName = comboBox1.SelectedItem.ToString();
                    serialPort1.Open();
                }
                catch (Exception Ex)
                {
                    textBox1.AppendText("Open Port at" + comboBox1.SelectedItem.ToString() + " error \r\n");
                    textBox1.AppendText(Ex.Message + " \r\n");
                    return;
                }

                // Serial port is NOT opened
                button1.Text = "Close Port";
                textBox1.AppendText("Start at " + DateTime.Now.ToString() + "\r\n");
            }
        }

        // 读取音乐文件
        private void button2_Click_1(object sender, EventArgs e)
        {
            nodeStruct newValue = new nodeStruct();

            if (openFileDialog1.ShowDialog() == DialogResult.OK)
            {

                FileStream fs = new FileStream(openFileDialog1.FileName, FileMode.Open, FileAccess.Read);

                StreamReader sr = new StreamReader(fs);
                //使用StreamReader类来读取文件
                sr.BaseStream.Seek(0, SeekOrigin.Begin);
                // 从数据流中读取每一行，直到文件的最后一行
                string tmp = sr.ReadLine();
                int MiniDelay = 65535;

                while (tmp != null)
                {
                    // 只有带有 Note_on_c 字样的才包含数据信息
                    if (tmp.IndexOf("Note_on_c") > 0)
                    {
                        // 数据使用“,” 分割，这里拆解成字符串
                        string[] arr = tmp.Split(',');

                        // 只记录敲击发生的数据，不记录停止的数据
                        if (Convert.ToInt32(arr[5]) != 0)
                        {
                            newValue.time = Convert.ToInt32(arr[1]);

                            // 转化为 LED 的位置号
                            for (int i = 0; i < LEDCOUNTER; i++) {
                                if (Convert.ToInt32(arr[4]) == NoteToOrder[i]) {
                                    newValue.note = i;
                                    break;
                                }
                            }
                            
                            if (NoteList.Count > 0)
                            {
                                MiniDelay = Math.Min(MiniDelay, newValue.time - NoteList[NoteList.Count - 1].time);
                                if (MiniDelay == 0) {
                                    Thread.Sleep(100);
                                }
                            }
                            if (NoteList.Count == 0) {
                                newValue.delayunit = 0;
                            }
                            NoteList.Add(newValue);
                        }

                        

                    }
                    tmp = sr.ReadLine();
                }
                //关闭此StreamReader对象
                sr.Close();
                fs.Close();

                textBox1.AppendText("Total " + NoteList.Count.ToString() + "/Mininal delay " + MiniDelay.ToString() + "\r\n");


                // 给歌曲信息赋值间隔数据
                for (int i = 1; i < NoteList.Count; i++) {
                    nodeStruct tmpNode = NoteList[i];
                    tmpNode.delayunit = (NoteList[i].time - NoteList[i - 1].time) / MiniDelay;
                    if (tmpNode.delayunit > 10) { tmpNode.delayunit = 9; }
                    NoteList[i] = tmpNode;
                }

                for (int i = 0; i < NoteList.Count;i++) {
                    textBox1.AppendText(i.ToString() + "  " +
                                        NoteList[i].time.ToString() + "   "+
                                        NoteList[i].note.ToString()+" "+
                                        NoteList[i].delayunit.ToString()+"\r\n"
                                        );
                }
                    textBox1.AppendText("File loaded\r\n");
                

            }
        }

        private void timer1_Tick_1(object sender, EventArgs e)
        {

            // 处理最后几个下落
            //LEDCOUNTER
            for (int i = 0; i < LEDCOUNTER; i++)
            {
                // 如果移动之前最低3Bits有1，那么清除之
                if ((Note[i] & 0b00000111) != 0)
                {
                    Note[i] = (byte)(Note[i] & 0b11111000);
                }
                Note[i] = (byte)((Note[i] & 0xFF) >> 1);
                //如果移动之后Bit2为1，那么最后3位置起来
                if ((Note[i] & 0b00000100) != 0)
                {
                    Note[i] = (byte)(Note[i] | 0b00000111);
                }
            }
            serialPort1.Write(Note, 0, LEDCOUNTER);
            string s = NoteIndex.ToString()+" ";
            for (int i = 0; i < LEDCOUNTER; i++)
            {
                s = s + Note[i].ToString("X2") + "   ";
            }
            textBox1.AppendText(s + "\r\n");

            // 如果当前已经是最后一个，那么就停止
            if (NoteIndex == NoteList.Count) {
                int v = 0;
                for (int i = 0; i < LEDCOUNTER; i++) {
                    v = v + Note[i];
                }
                if (v==0)
                {
                    textBox1.AppendText("End of the song\r\n");
                    timer1.Enabled = false;
                    return;
                }
                return;
            }

            // 如果到达时间
            if (Elsp >= NoteList[NoteIndex].delayunit)
            {
                Note[NoteList[NoteIndex].note] = (byte)(Note[NoteList[NoteIndex].note]|0x80);
                serialPort1.Write(Note, 0, LEDCOUNTER);
                NoteIndex++;
                Elsp = 0;
            }
            else
            {
                Elsp++;
            }

        }

        private void button3_Click(object sender, EventArgs e)
        {
            if (timer1.Enabled)
            {
                timer1.Enabled = false;
                button3.Text = "Start Send";
            }
            else
            {
                for (int i = 0; i < LEDCOUNTER; i++) {
                    Note[i] = 0xFF;
                    serialPort1.Write(Note, 0, LEDCOUNTER);
                    Thread.Sleep(100);
                }
                Thread.Sleep(200);

                for (int i = LEDCOUNTER-1; i >-1 ; i--)
                {
                    Note[i] = 0x00;
                    serialPort1.Write(Note, 0, LEDCOUNTER);
                    Thread.Sleep(100);
                }
                button3.Text = "Stop Send";
                timer1.Interval = ONEDELAY;
                timer1.Enabled = true;
                NoteIndex = 0;
                Elsp = 0;
            }

        }
    }
}
